"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiquidityPoolVenues = exports.SwapToLP = void 0;
const web3_js_1 = require("@solana/web3.js");
const core_1 = require("@jup-ag/core");
const cross_fetch_1 = __importDefault(require("cross-fetch"));
const raydium_sdk_1 = require("@raydium-io/raydium-sdk");
const spl_token_1 = require("@solana/spl-token");
const logger_1 = require("./logger");
const sdk_1 = require("@orca-so/sdk");
const decimal_js_1 = __importDefault(require("decimal.js"));
const logger = logger_1.Logger.from("sdk/swap");
const feeAccount = "6nYhSkKDU1X8ZuygrrYYdkU6jWT8mWWHbN2D3TvVrqxp";
const SOLANA_TOKEN_LIST_URL = 'https://raw.githubusercontent.com/solana-labs/token-list/main/src/tokens/solana.tokenlist.json';
const RAYDIUM_LP_URL = 'https://sdk.raydium.io/liquidity/mainnet.json';
const JUPITER_BASE_URL = 'https://quote-api.jup.ag/v1/quote';
/**
 * SwapToLP class for swapping SPL tokens to LP tokens through rebalancing and staking.
 */
class SwapToLP {
    /**
     * Constructor for the SwapToLP class. Builds and returns a SwapToLP object.
     */
    constructor(connection, jupiter) {
        this.connection = connection;
        this.jupiter = jupiter;
    }
    /**
     * Initializes a SwapToLP object.
     */
    static build(connection, user) {
        return __awaiter(this, void 0, void 0, function* () {
            // 1. Fetch fee token accounts
            const feeAccounts = yield (0, core_1.getPlatformFeeAccounts)(connection, new web3_js_1.PublicKey(feeAccount));
            const platformFeeAndAccounts = {
                feeBps: 50,
                feeAccounts: feeAccounts
            };
            // 2. Instantiate jupiter instance
            const jupiter = yield core_1.Jupiter.load({
                connection,
                cluster: 'mainnet-beta',
                user: user,
                platformFeeAndAccounts
            });
            // 3. Instantiate and return SwapToLP object
            return new SwapToLP(connection, jupiter);
        });
    }
    /**
     * Generates a set of unsigned transactions for swapping a single SPL token to base/quote tokens required for LP. The transactions include setup transactions for the output tokens based on the LP token's base and quote mints.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @param liquidityPoolVenue - venue of liquidity pool
     * @param slippageTolerancePercent - slippage tolerance in percent, between 0 and 100
     */
    swap(inputTokenAmount, inputTokenMint, lpTokenMint, liquidityPoolVenue = LiquidityPoolVenues.RAYDIUM, slippageTolerancePercent = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug(`inputTokenMint: ${inputTokenMint}`);
            logger.debug(`lpTokenMint: ${lpTokenMint}`);
            logger.debug(`inputTokenAmount: ${inputTokenAmount}`);
            // Validate input is greater than 0
            logger.assertArgument(inputTokenAmount > 0, 'inputTokenAmount must be greater than 0', "inputTokenAmount", inputTokenAmount);
            // Validate slippage tolerance is between 0 and 100
            logger.assertArgument(slippageTolerancePercent >= 0 && slippageTolerancePercent <= 100, 'slippageTolerancePercent must be between 0 and 100', "slippageTolerancePercent", slippageTolerancePercent);
            var transactions = [];
            // 1. Fetch pool and token info
            var baseMint;
            var quoteMint;
            switch (liquidityPoolVenue) {
                case LiquidityPoolVenues.RAYDIUM: {
                    const poolInfo = yield this._getRaydiumPoolInfo(lpTokenMint);
                    logger.debug('pool info:', poolInfo);
                    baseMint = poolInfo.baseMint;
                    quoteMint = poolInfo.quoteMint;
                    break;
                }
                case LiquidityPoolVenues.ORCA: {
                    try {
                        const poolInfo = yield this._getOrcaPoolInfo(lpTokenMint);
                        baseMint = poolInfo.getTokenA().mint.toBase58();
                        quoteMint = poolInfo.getTokenB().mint.toBase58();
                    }
                    catch (_a) {
                        logger.throwArgumentError('no orca pool found with mint ' + lpTokenMint.toBase58(), logger_1.Logger.errors.INVALID_ARGUMENT, { message: 'no orca pool found with mint ' + lpTokenMint.toBase58() });
                    }
                    break;
                }
                default: {
                    logger.throwArgumentError('cannot swap to required tokens as liquidity pool venue is not handled', logger_1.Logger.errors.UNSUPPORTED_VENUE, { message: 'liquidity pool venue is not handled' });
                    break;
                }
            }
            const baseInfo = yield this._getTokenInfo(baseMint);
            logger.debug('baseInfo:', baseInfo);
            const quoteInfo = yield this._getTokenInfo(quoteMint);
            logger.debug('quoteInfo:', quoteInfo);
            const inputTokenInfo = yield this._getTokenInfo(inputTokenMint.toBase58());
            const inputTokenPrice = yield this.getJupiterTokenPrice(inputTokenMint.toBase58(), inputTokenInfo.decimals);
            logger.debug('inputTokenInfo:', inputTokenInfo);
            logger.debug('inputTokenPrice:', inputTokenPrice);
            const basePrice = yield this.getJupiterTokenPrice(baseMint, baseInfo.decimals);
            const quotePrice = yield this.getJupiterTokenPrice(quoteMint, quoteInfo.decimals);
            logger.debug('basePrice:', basePrice);
            logger.debug('quotePrice:', quotePrice);
            var baseRoute = undefined;
            var quoteRoute = undefined;
            // 2. Convert 1/2 inputTokenAmount in USD to token A
            // 2. Skip if input token = base token
            if (inputTokenMint.toBase58() !== baseMint) {
                // 2.1 Find routes for swapping input to base token
                var baseRoutes = yield this.jupiter.computeRoutes({
                    inputMint: inputTokenMint,
                    outputMint: new web3_js_1.PublicKey(baseMint),
                    inputAmount: Math.floor(inputTokenAmount / 2),
                    slippage: slippageTolerancePercent,
                    forceFetch: true
                });
                logger.debug('baseRoutes:', baseRoutes.routesInfos.length);
                baseRoutes.routesInfos = baseRoutes.routesInfos.sort((a, b) => a.outAmountWithSlippage - b.outAmountWithSlippage);
                // 2.2 Validate routes are present
                if (baseRoutes.routesInfos.length === 0) {
                    return logger.throwError("No base routes found", logger_1.Logger.errors.JUPITER_ERROR, { message: 'No base routes found' });
                }
                // 2.3 Attempt to compile route until one is found
                var baseExchange = undefined;
                while (baseExchange === undefined) {
                    // 2.4 Select a route, starting with the best priced route
                    for (var i = 0; i < baseRoutes.routesInfos.length; i++) {
                        baseRoute = baseRoutes.routesInfos[i];
                        // 2.5 Try generate transactions
                        try {
                            baseExchange = yield this.jupiter.exchange({ routeInfo: baseRoute });
                        }
                        catch (e) {
                            logger.warn('Error generating base route transactions', e);
                            i++;
                        }
                    }
                }
                // 2.6 Add transactions - NOTE: ordering is important
                // 2.6.1 Add setup transaction
                if (baseExchange.transactions.setupTransaction) {
                    transactions.push(baseExchange.transactions.setupTransaction);
                }
                // 2.6.2 Add swap transaction
                if (baseExchange.transactions.swapTransaction) {
                    transactions.push(baseExchange.transactions.swapTransaction);
                }
                // 2.6.3 Add cleanup transaction
                if (baseExchange.transactions.cleanupTransaction) {
                    transactions.push(baseExchange.transactions.cleanupTransaction);
                }
            }
            logger.debug(`Generated ${transactions.length} base transactions`);
            // 3. Convert 1/2 inputTokenAmount in USD to token B
            // 3. Skip if input token = quote token
            if (inputTokenMint.toBase58() !== quoteMint) {
                // 3.1 Find routes for swapping input to quote token
                const quoteRoutes = yield this.jupiter.computeRoutes({
                    inputMint: inputTokenMint,
                    outputMint: new web3_js_1.PublicKey(quoteMint),
                    inputAmount: Math.floor(inputTokenAmount / 2),
                    slippage: slippageTolerancePercent,
                    forceFetch: true
                });
                logger.debug(`quoteRoutes: ${quoteRoutes.routesInfos.length}`);
                quoteRoutes.routesInfos = quoteRoutes.routesInfos.sort((a, b) => a.outAmountWithSlippage - b.outAmountWithSlippage);
                // 3.2 Validate routes are present
                if (quoteRoutes.routesInfos.length === 0) {
                    return logger.throwError("No quote routes found", logger_1.Logger.errors.JUPITER_ERROR, { message: 'No quote routes found' });
                }
                // 3.3 Attempt to compile route until one is found
                var quoteExchange = undefined;
                while (quoteExchange === undefined) {
                    // 3.4 Select a route, starting with the best priced route
                    for (var i = 0; i < quoteRoutes.routesInfos.length; i++) {
                        quoteRoute = quoteRoutes.routesInfos[i];
                        // 3.5 Try generate transactions
                        try {
                            quoteExchange = yield this.jupiter.exchange({ routeInfo: quoteRoute });
                        }
                        catch (e) {
                            logger.warn('Error generating quote route transactions', e);
                            i++;
                        }
                    }
                }
                // 3.6 Add transactions - NOTE: ordering is important
                // 3.6.1 Add setup transaction
                if (quoteExchange.transactions.setupTransaction) {
                    transactions.push(quoteExchange.transactions.setupTransaction);
                }
                // 3.6.2 Add swap transaction
                if (quoteExchange.transactions.swapTransaction) {
                    transactions.push(quoteExchange.transactions.swapTransaction);
                }
                // 3.6.3 Add cleanup transaction
                if (quoteExchange.transactions.cleanupTransaction) {
                    transactions.push(quoteExchange.transactions.cleanupTransaction);
                }
            }
            logger.debug(`Generated ${transactions.length} transactions in total`);
            return {
                transactions,
                prices: {
                    inputTokenPrice,
                    basePrice,
                    quotePrice
                },
                tokenInfo: {
                    inputTokenInfo,
                    baseInfo,
                    quoteInfo
                },
                baseRoute,
                quoteRoute
            };
        });
    }
    /**
     * Generate a transaction to add liquidity to a Raydium LP and if required to open the LP token account.
     * @param lpTokenMint - mint of lp token to receive
     * @param poolInfo - Raydium pool info
     * @param tokenAAmount - raw amount of base token to stake in LP
     * @param user - Public key of owner of LP token account
     * @param connection - web.js connection
     * @param addLiquiditySlippagePercent - slippage tolerance for adding liquidity in percent, between 0 and 100
     */
    _createAddLiquidityRaydiumTx(lpTokenMint, poolInfo, tokenAAmount, user, connection, addLiquiditySlippagePercent = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            // 1. Create undefined transaction instruction var
            var tokenAccountIx = undefined;
            // 2. Find user's token account based on LP mint
            const usersLpTokenAddress = yield raydium_sdk_1.Spl.getAssociatedTokenAccount({ mint: lpTokenMint, owner: user });
            // 3. Fetch account info for user's 
            const associatedAccount = connection
                ? yield connection.getAccountInfo(usersLpTokenAddress)
                : yield this.connection.getAccountInfo(usersLpTokenAddress);
            if (associatedAccount === null) {
                tokenAccountIx = yield spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, lpTokenMint, usersLpTokenAddress, user, user);
            }
            // 4. Convert json to pool keys object
            const poolKeys = (0, raydium_sdk_1.jsonInfo2PoolKeys)(poolInfo);
            logger.debug('poolKeys:', poolKeys);
            // 5. Get pool info
            const raydiumPoolInfo = yield raydium_sdk_1.Liquidity.fetchInfo({ connection: connection ? connection : this.connection, poolKeys });
            logger.debug('raydiumPoolInfo:', raydiumPoolInfo);
            const baseCurrency = new raydium_sdk_1.Token(poolInfo.baseMint, raydiumPoolInfo.baseDecimals);
            const quoteCurrency = new raydium_sdk_1.Token(poolInfo.quoteMint, raydiumPoolInfo.quoteDecimals);
            // 6. Convert base amount to quote amount
            const baseAmount = new raydium_sdk_1.TokenAmount(baseCurrency, Math.floor(tokenAAmount), true);
            const quoteAmount = raydium_sdk_1.Liquidity.computeAnotherAmount({
                poolKeys,
                poolInfo: raydiumPoolInfo,
                amount: baseAmount,
                anotherCurrency: quoteCurrency,
                slippage: new raydium_sdk_1.Percent(addLiquiditySlippagePercent),
            });
            logger.debug('input amount:', tokenAAmount);
            // 7. Generate instruction to add liquidity
            const ix = yield raydium_sdk_1.Liquidity.makeAddLiquidityInstruction({
                poolKeys: poolKeys,
                userKeys: {
                    baseTokenAccount: yield raydium_sdk_1.Spl.getAssociatedTokenAccount({ mint: new web3_js_1.PublicKey(poolInfo.baseMint), owner: user }),
                    quoteTokenAccount: yield raydium_sdk_1.Spl.getAssociatedTokenAccount({ mint: new web3_js_1.PublicKey(poolInfo.quoteMint), owner: user }),
                    lpTokenAccount: usersLpTokenAddress,
                    owner: user
                },
                baseAmountIn: baseAmount.raw,
                quoteAmountIn: quoteAmount.maxAnotherAmount.raw,
                fixedSide: "base"
            });
            // 8. Return new transaction
            var addLiquidityTx = new web3_js_1.Transaction();
            if (tokenAccountIx !== undefined) {
                addLiquidityTx.add(tokenAccountIx);
            }
            addLiquidityTx.add(ix);
            addLiquidityTx.recentBlockhash = (yield this.connection.getLatestBlockhash()).blockhash;
            addLiquidityTx.feePayer = user;
            return addLiquidityTx;
        });
    }
    /**
     * Generate a transaction to add liquidity to an Orca LP and if required to open the LP token account.
     * @param lpTokenMint - mint of lp token to receive
     * @param poolInfo - Orce pool info
     * @param tokenAAmount - raw amount of base token to stake in LP
     * @param tokenBAmount - raw amount of quote token to stake in LP
     * @param user - Public key of owner of LP token account
     * @param connection - web.js connection
     */
    _createAddLiquidityOrcaTx(lpTokenMint, tokenAAmount, tokenBAmount, user, addLiquiditySlippagePercent = 1) {
        return __awaiter(this, void 0, void 0, function* () {
            logger.debug('_createAddLiquidityOrcaTx');
            const poolInfo = yield this._getOrcaPoolInfo(lpTokenMint);
            const baseAmount = new decimal_js_1.default(tokenAAmount);
            logger.debug('baseAmount:', baseAmount.toString());
            const quoteAmount = new decimal_js_1.default(tokenBAmount);
            logger.debug('quoteAmount:', quoteAmount.toString());
            const { maxTokenAIn, maxTokenBIn, minPoolTokenAmountOut } = yield poolInfo.getDepositQuote(baseAmount, quoteAmount, new decimal_js_1.default(addLiquiditySlippagePercent));
            logger.debug('maxTokenAIn:', maxTokenAIn.toNumber());
            logger.debug('maxTokenBIn:', maxTokenBIn.toNumber());
            logger.debug('minPoolTokenAmountOut:', minPoolTokenAmountOut.toNumber());
            const poolDepositPayload = yield poolInfo.deposit(user, maxTokenAIn, maxTokenBIn, minPoolTokenAmountOut);
            logger.debug('poolDepositPayload:', poolDepositPayload);
            poolDepositPayload.transaction.recentBlockhash = (yield this.connection.getLatestBlockhash()).blockhash;
            poolDepositPayload.transaction.feePayer = user;
            for (const signer of poolDepositPayload.signers) {
                poolDepositPayload.transaction.sign(signer);
            }
            return poolDepositPayload.transaction;
        });
    }
    /**
     * Helper method to bundling `swapToBaseQuote` and `addLiquidity` together.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @returns Array of transactions
     */
    swapAndAddLiquidity(inputTokenAmount, inputTokenMint, lpTokenMint, user, connection, liquidityPoolVenue = LiquidityPoolVenues.RAYDIUM) {
        return __awaiter(this, void 0, void 0, function* () {
            // 1. Swap input token to base and quote tokens for LP
            const { transactions, prices, tokenInfo, baseRoute, quoteRoute } = yield this.swap(inputTokenAmount, inputTokenMint, lpTokenMint, liquidityPoolVenue);
            // 1.1. Find base and quote tokens for LP
            var baseMint = undefined;
            var quoteMint = undefined;
            var poolInfo = undefined;
            switch (liquidityPoolVenue) {
                case LiquidityPoolVenues.RAYDIUM: {
                    poolInfo = yield this._getRaydiumPoolInfo(lpTokenMint);
                    baseMint = poolInfo.baseMint;
                    quoteMint = poolInfo.quoteMint;
                    break;
                }
                case LiquidityPoolVenues.ORCA: {
                    poolInfo = yield this._getOrcaPoolInfo(lpTokenMint);
                    baseMint = poolInfo.getTokenA().mint.toBase58();
                    quoteMint = poolInfo.getTokenB().mint.toBase58();
                    break;
                }
                default: {
                    logger.throwArgumentError('cannot swap to required tokens as liquidity pool venue is not handled', logger_1.Logger.errors.UNSUPPORTED_VENUE, { message: 'liquidity pool venue is not handled' });
                    break;
                }
            }
            // 2. Calculate quote amount to stake
            // 2.1 Validate if base is input token, if not, calculate base out amount 
            const baseAmountAfterSlippage = inputTokenMint.equals(new web3_js_1.PublicKey(baseMint))
                ? (inputTokenAmount / 2) / (Math.pow(10, tokenInfo.baseInfo.decimals))
                : baseRoute.outAmountWithSlippage / (Math.pow(10, tokenInfo.baseInfo.decimals));
            logger.debug('baseOutAmount:', inputTokenMint.equals(new web3_js_1.PublicKey(baseMint)) ? (inputTokenAmount / 2) : baseRoute.outAmountWithSlippage);
            logger.debug('baseDecimals:', tokenInfo.baseInfo.decimals);
            logger.debug('baseAmountAfterSlippage:', baseAmountAfterSlippage);
            // 2.2 Validate if quote is input token, if not, calculate quote out amount 
            const quoteAmountAfterSlippage = inputTokenMint.equals(new web3_js_1.PublicKey(quoteMint))
                ? (inputTokenAmount / 2) / (Math.pow(10, tokenInfo.quoteInfo.decimals))
                : quoteRoute.outAmountWithSlippage / (Math.pow(10, tokenInfo.quoteInfo.decimals));
            logger.debug('quoteOutAmount:', inputTokenMint.equals(new web3_js_1.PublicKey(quoteMint)) ? (inputTokenAmount / 2) : quoteRoute.outAmountWithSlippage);
            logger.debug('quoteDecimals:', tokenInfo.quoteInfo.decimals);
            logger.debug('quoteAmountAfterSlippage:', quoteAmountAfterSlippage);
            // 2.3 Convert to USD
            const baseAmountAfterSlippageUsd = prices.basePrice * baseAmountAfterSlippage;
            const quoteAmountAfterSlippageUsd = prices.quotePrice * quoteAmountAfterSlippage;
            logger.debug('basePrice:', prices.basePrice);
            logger.debug('quotePrice:', prices.quotePrice);
            logger.debug('baseAmountAfterSlippageUsd:', baseAmountAfterSlippageUsd);
            logger.debug('quoteAmountAfterSlippageUsd:', quoteAmountAfterSlippageUsd);
            // 2.4 Calculate smaller size in USD
            const isBaseSmallerThanQuote = baseAmountAfterSlippageUsd < quoteAmountAfterSlippageUsd;
            logger.debug('isBaseSmallerThanQuote:', isBaseSmallerThanQuote);
            // 2.5 Convert smaller size to token amount
            const baseAmount = ((isBaseSmallerThanQuote ? baseAmountAfterSlippageUsd : quoteAmountAfterSlippageUsd) / prices.basePrice) * (Math.pow(10, tokenInfo.baseInfo.decimals));
            logger.debug('baseAmount:', baseAmount);
            // 3. Create add liquidity tx
            var addLiquidityTx;
            switch (liquidityPoolVenue) {
                case LiquidityPoolVenues.RAYDIUM: {
                    addLiquidityTx = yield this._createAddLiquidityRaydiumTx(lpTokenMint, 
                    // @ts-expect-error
                    poolInfo, baseAmount, user, connection);
                    break;
                }
                case LiquidityPoolVenues.ORCA: {
                    addLiquidityTx = yield this._createAddLiquidityOrcaTx(lpTokenMint, baseAmountAfterSlippage, quoteAmountAfterSlippage, user);
                    break;
                }
                default: {
                    logger.throwArgumentError('cannot add liquidity as liquidity pool venue is not handled', logger_1.Logger.errors.UNSUPPORTED_VENUE, { message: 'liquidity pool venue is not handled' });
                    break;
                }
            }
            transactions.push(addLiquidityTx);
            return {
                transactions: transactions,
                routes: {
                    baseRoute,
                    quoteRoute
                },
                prices,
                tokenInfo
            };
        });
    }
    /**
     * Get route info.
     * @param inputTokenAmount - raw amount to swap
     * @param inputTokenMint - mint of token to swap
     * @param lpTokenMint - mint of lp token to receive
     * @param liquidityPoolVenue - venue of liquidity pool
     */
    getRouteInfo(inputTokenAmount = 2, inputTokenMint, lpTokenMint, liquidityPoolVenue = LiquidityPoolVenues.RAYDIUM) {
        return __awaiter(this, void 0, void 0, function* () {
            var baseRoute = undefined;
            var quoteRoute = undefined;
            var baseMint;
            var quoteMint;
            switch (liquidityPoolVenue) {
                case LiquidityPoolVenues.RAYDIUM: {
                    const poolInfo = yield this._getRaydiumPoolInfo(lpTokenMint);
                    logger.debug('pool info:', poolInfo);
                    baseMint = poolInfo.baseMint;
                    quoteMint = poolInfo.quoteMint;
                    break;
                }
                case LiquidityPoolVenues.ORCA: {
                    const poolInfo = yield this._getOrcaPoolInfo(lpTokenMint);
                    logger.debug('pool info:', poolInfo);
                    baseMint = poolInfo.getTokenA().mint.toBase58();
                    quoteMint = poolInfo.getTokenB().mint.toBase58();
                    break;
                }
                default: {
                    logger.throwArgumentError('cannot swap to required tokens as liquidity pool venue is not handled', logger_1.Logger.errors.UNSUPPORTED_VENUE, { message: 'liquidity pool venue is not handled' });
                    break;
                }
            }
            if (inputTokenMint.toBase58() !== baseMint) {
                const baseRoutes = yield this.jupiter.computeRoutes({
                    inputMint: inputTokenMint,
                    outputMint: new web3_js_1.PublicKey(baseMint),
                    inputAmount: inputTokenAmount / 2,
                    slippage: 1,
                    forceFetch: true
                });
                if (baseRoutes.routesInfos.length > 0) {
                    baseRoute = baseRoutes.routesInfos[0];
                }
            }
            if (inputTokenMint.toBase58() !== quoteMint) {
                const quoteRoutes = yield this.jupiter.computeRoutes({
                    inputMint: inputTokenMint,
                    outputMint: new web3_js_1.PublicKey(quoteMint),
                    inputAmount: inputTokenAmount / 2,
                    slippage: 1,
                    forceFetch: true
                });
                if (quoteRoutes.routesInfos.length > 0) {
                    quoteRoute = quoteRoutes.routesInfos[0];
                }
            }
            return {
                baseRoute,
                quoteRoute
            };
        });
    }
    /**
     * Get pool info for Raydium LP token mint.
     * @param lpTokenMint - LP token mint
     */
    _getRaydiumPoolInfo(lpTokenMint) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield (0, cross_fetch_1.default)(RAYDIUM_LP_URL);
            const pools = yield response.json();
            const allPools = pools.official.concat(pools.unOfficial);
            const pool = allPools.find(p => p.lpMint === lpTokenMint.toBase58());
            if (!pool) {
                throw new Error('Pool not found');
            }
            return pool;
        });
    }
    /**
     * Get pool info for Orca LP token mint.
     * @param lpTokenMint - LP token mint
     */
    _getOrcaPoolInfo(lpTokenMint) {
        return __awaiter(this, void 0, void 0, function* () {
            const orca = (0, sdk_1.getOrca)(this.connection);
            const lpMints = Object.keys(sdk_1.OrcaPoolConfig);
            var orcaPools = lpMints.map(lpMint => {
                var orcaPool = orca.getPool(sdk_1.OrcaPoolConfig[lpMint]);
                return {
                    orcaPool,
                    lpMint: orcaPool.getPoolTokenMint()
                };
            });
            const poolMint = orcaPools.find(mint => mint.lpMint.toBase58() === lpTokenMint.toBase58());
            logger.debug('pool mint:', poolMint);
            logger.debug('orcaPool:', poolMint.orcaPool);
            return poolMint.orcaPool;
        });
    }
    /**
     * Get token info for a given mint.
     * @param tokenMint - token mint
     */
    _getTokenInfo(tokenMint) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield (0, cross_fetch_1.default)(SOLANA_TOKEN_LIST_URL);
            const tokens = yield response.json();
            const tokenInfo = tokens.tokens.find(p => p.address === tokenMint);
            return tokenInfo;
        });
    }
    /**
     * Get token price for a given mint.
     * @param tokenMint - mint of token price to lookup
     * @param tokenDecimals - token decimals
     */
    getJupiterTokenPrice(tokenMint, tokenDecimals) {
        return __awaiter(this, void 0, void 0, function* () {
            if (tokenMint === 'EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v') {
                return 1;
            }
            const jupiterURL = `${JUPITER_BASE_URL}?inputMint=${tokenMint}&outputMint=EPjFWdd5AufqSSqeM2qN1xzybapC8G4wEGGkZwyTDt1v&amount=${Math.pow(10, tokenDecimals)}`;
            logger.debug('jupiter URL:', jupiterURL);
            const response = yield (0, cross_fetch_1.default)(jupiterURL);
            const jsonReponse = yield response.json();
            logger.debug('jupiter token price info:', jsonReponse);
            const outAmount = jsonReponse.data[0].outAmount / 1e6;
            return outAmount;
        });
    }
}
exports.SwapToLP = SwapToLP;
var LiquidityPoolVenues;
(function (LiquidityPoolVenues) {
    LiquidityPoolVenues["RAYDIUM"] = "RAYDIUM";
    LiquidityPoolVenues["ORCA"] = "ORCA";
})(LiquidityPoolVenues = exports.LiquidityPoolVenues || (exports.LiquidityPoolVenues = {}));
